'''
==================================================================
    IMPORTS
==================================================================
'''
# Python imports
import random, os

# EventScripts Imports
import cfglib, es, gamethread, langlib, popuplib, services

#import os.path
from configobj import ConfigObj
from playerlib import getPlayer
from weaponlib import getWeaponList

# Import SPE
try:
   import spe
   from spe.games import cstrike 
except ImportError:
    es.dbgmsg(0, " ")
    es.dbgmsg(0, " ")
    es.dbgmsg(0, "* ========================== gDeathMatch ========================== *")
    es.dbgmsg(0, "*             Your server does not meet the requirements!")
    es.dbgmsg(0, "*                     The addon will be unloaded.")
    es.dbgmsg(0, "* ")
    es.dbgmsg(0, "*  To meet the requirements you will need to Download SPE:")
    es.dbgmsg(0, "*    -> http://forums.eventscripts.com/viewtopic.php?t=29657#269624")
    es.dbgmsg(0, "* ================================================================= *")
    es.dbgmsg(0, " ")
    es.dbgmsg(0, " ")
''' ; end ; '''


'''
==============================================================================
   ADDON INFO
==============================================================================
'''
addon = {
    'name'       : 'gDeathMatch',
    'version'    : '1.0',
    'date'       : '11.08.2011',
    'author'     : 'G l a z z',
    'description': 'DeathMatch addon ...',
    'url'        : '',
    'basename'   : 'gdeathmatch'
}
info = es.AddonInfo()
for i in addon: info[i] = addon[i]
es.ServerVar(info['basename'], info['version'], info['description']).makepublic()
''' ; end ; '''


'''
==============================================================================
   PATH
==============================================================================
'''
dmPath = str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/').split('/')[~0]
''' ; end ; '''


'''
==============================================================================
    CONFIG VARIABLES
==============================================================================
'''
dm_enable                   = es.ServerVar('dm_enable')
dm_guns_menu                = es.ServerVar('dm_guns_menu')
dm_weapon_remover           = es.ServerVar('dm_weapon_remover')
dm_allow_roundend           = es.ServerVar('dm_allow_roundend')
dm_allow_buyzones           = es.ServerVar('dm_allow_buyzones')
dm_allow_map_objectives     = es.ServerVar('dm_allow_map_objectives')
dm_p_weapons                = es.ServerVar('dm_p_weapons')
dm_s_weapons                = es.ServerVar('dm_s_weapons')
dm_restrict                 = es.ServerVar('dm_restrict')
dm_amount_health            = es.ServerVar('dm_amount_health')
dm_amount_armor             = es.ServerVar('dm_amount_armor')
dm_give_hegrenade           = es.ServerVar('dm_give_hegrenade')
dm_give_flashbang           = es.ServerVar('dm_give_flashbang')
dm_give_smokegrenade        = es.ServerVar('dm_give_smokegrenade')
dm_give_unlimitednade       = es.ServerVar('dm_give_unlimitednade')
dm_give_unlimitedammo       = es.ServerVar('dm_give_unlimitedammo')
dm_give_ngvision            = es.ServerVar('dm_give_ngvision')
dm_spawn_method             = es.ServerVar('dm_spawn_method')
dm_spawn_deadtime           = es.ServerVar('dm_spawn_deadtime')
dm_spawn_protection         = es.ServerVar('dm_spawn_protection')
dm_spawn_protection_time    = es.ServerVar('dm_spawn_protection_time')
dm_spawn_protection_type    = es.ServerVar('dm_spawn_protection_type')
dm_spawn_protection_alpha   = es.ServerVar('dm_spawn_protection_alpha')
dm_spawn_protection_health  = es.ServerVar('dm_spawn_protection_health')
dm_dissolver_time           = es.ServerVar('dm_dissolver_time')
dm_dissolver_type           = es.ServerVar('dm_dissolver_type')
''' ; end ; '''


'''
==============================================================================
   VARIABLES and DICTs
==============================================================================
'''
# Variables.
CurMap = es.ServerVar('eventscripts_currentmap')
AllowRespawn = True

# DICTs.
Language = None
LoadedAddons = []
MapsCfg = None
myPopups = {}
Players = {}
Spawnpoints = {}
Weapons = {
    'weapon_m4a1'      : 90,
    'weapon_ak47'      : 90,
    'weapon_awp'       : 30,
    'weapon_scout'     : 90,
    'weapon_sg552'     : 90,
    'weapon_galil'     : 90,
    'weapon_famas'     : 90,
    'weapon_sg550'     : 90,
    'weapon_aug'       : 90,
    'weapon_m249'      : 200,
    'weapon_g3sg1'     : 90,
    'weapon_ump45'     : 100,
    'weapon_mp5navy'   : 120,
    'weapon_m3'        : 32,
    'weapon_xm1014'    : 32,
    'weapon_tmp'       : 120,
    'weapon_mac10'     : 120,
    'weapon_p90'       : 100,
    'weapon_usp'       : 100,
    'weapon_glock'     : 100,
    'weapon_deagle'    : 35,
    'weapon_p228'      : 52,
    'weapon_elite'     : 120,
    'weapon_fiveseven' : 100
}
''' ; end ; '''


'''
==============================================================================
   DELAY MANAGEMENT class
==============================================================================
'''
class DelayManager:
    def __init__(self):
        self.__call__ = self.delayed
        self.delays = {}

    def delayed(self, seconds, name, func, args=()):
        if name in self.delays: self.cancel(name)
        if not isinstance(args, tuple):
            args = (args,)
        self.delays[name] = (func, args)
       
        gamethread.delayedname(seconds, name, self.callback, name)
       
    def callback(self, name):
        func, args = self.delays[name]
        del self.delays[name]
        func(*args)

    def cancel(self, name):
        gamethread.cancelDelayed(name)
        if name in self.delays:
            del self.delays[name]

    def cancelAll(self):
        for name in self.delays:
            gamethread.cancelDelayed(name)
        self.delays = {}
''' ; end - DelayManager() ; '''

'''
==============================================================================
   PLAYER MANAGEMENT class
==============================================================================
'''
class PlayerManagement:
    '''
    | Initiate...
    '''
    def __init__(self, userid):
        self.userid    = userid
        self.team      = 0
        self.pweapon   = 0
        self.sweapon   = 0
        self.gunmenu   = 1
        self.protected = 0
        self.used      = 0

    '''
    | Reset..
    '''
    def Reset(self):
        self.team      = 0
        self.pweapon   = 0
        self.sweapon   = 0
        self.gunmenu   = 1
        self.protected = 0
        self.used      = 0

    '''
    | Strip player weapons.
    '''
    def StripWeapons(self):
        # UserID.
        UserID = self.userid

        # Strip player weapons.
        es.server.queuecmd('es_xfire %i player_weaponstrip kill' % UserID)
        es.server.queuecmd('es_xgive %i player_weaponstrip' % UserID)
        es.server.queuecmd('es_xfire %i player_weaponstrip strip' % UserID)

    '''
    | Remove player weapons, primary or secondary
    '''
    def RemoveWeapons(self, type):
        handle = es.getplayerhandle(self.userid)
        for weapon in getWeaponList('#%s' % type):
            for index in weapon.indexlist:
                if es.getindexprop(index, 'CBaseEntity.m_hOwnerEntity') == handle:
                    es.server.cmd('es_xremove %s' % index)

    '''
    | Give selected weapons to a player.
    '''
    def GiveWeapons(self):
        # UserID.
        UserID = self.userid

        # Give weapons.
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, self.pweapon))
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, self.sweapon))

    '''
    | Give random weapons to a player.
    '''
    def GiveWeaponsRandom(self):
        # UserID.
        UserID = self.userid

        # Give weapons.
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(str(dm_p_weapons).split())))
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(str(dm_s_weapons).split())))

    '''
    | Gives grenades and kevlar to a player.
    '''
    def GiveEquipment(self):
        # UserID.
        UserID = self.userid

        # Give a knife!
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_knife' % UserID)

        # Give a HE grenade?
        if int(dm_give_hegrenade) == 1: gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_hegrenade' % UserID)

        # Give FlashBang grenades?
        if int(dm_give_flashbang) == 1:
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_flashbang' % UserID)
        elif int(dm_give_flashbang) >= 2:
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_flashbang' % UserID)
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_flashbang' % UserID)

        # Give a Smoke grenade?
        if int(dm_give_smokegrenade) == 1: gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_smokegrenade' % UserID)

        # Give nightvision?
        if int(dm_give_ngvision) == 1: gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i item_nvgs' % UserID)

    '''
    | Check player weapons.
    '''
    def CheckWeapons(self):
        # UserID.
        UserID = self.userid

        # Do we have any weapons saved?
        if self.pweapon not in Weapons and self.sweapon not in Weapons:
            # Primary weapon.
            if MapsCfg['weapons']['p_weapons'] != 'None' or MapsCfg['weapons']['p_weapons'] != 'False': self.pweapon = MapsCfg['weapons']['p_weapons']

            # Secondary weapon.
            if MapsCfg['weapons']['s_weapons'] != 'None' or MapsCfg['weapons']['s_weapons'] != 'False': self.sweapon = MapsCfg['weapons']['s_weapons']

        # Give weapons.
        if self.pweapon:
            # Random weapon?
            if len(self.pweapon.split()) > 1:
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(self.pweapon.split())))
            else:
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, self.pweapon))
        if self.sweapon:
            # Random weapon?
            if len(self.sweapon.split()) > 1:
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(self.sweapon.split())))
            else:
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, self.sweapon))

    '''
    | When a player spawns.
    '''
    def Spawn(self):
        # UserID.
        UserID = self.userid

        # Strip player weapons.
        self.StripWeapons()

        # Amount of Health and Armor.
        player = getPlayer(UserID)
        player.health = dm_amount_health
        player.armor = dm_amount_armor

        # Guns menu is enabled?
        if int(dm_guns_menu) == 1:
            # Is player a Bot?
            if es.isbot(UserID):
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(str(dm_p_weapons).split())))
                gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, random.choice(str(dm_s_weapons).split())))
            else:
                # 1: New weapons every time
                if self.gunmenu == 1:
                    myPopups['dm_gunsmenu'].unsend(UserID)
                    gamethread.delayed(0.1, myPopups['dm_gunsmenu'].send, UserID)

                # 2: Same weapons every time
                elif self.gunmenu == 2:
                    self.GiveWeapons()

                # 3: Random weapons every time
                elif self.gunmenu == 3:
                    self.GiveWeaponsRandom()
        else:
            # Guns Menu is disabled, so let's check the weapons..
            self.CheckWeapons()

        # Give equipment.
        self.GiveEquipment()

        # Spawn protection.
        self.AddProtection()

    '''
    | Gives spawn protection to a player.
    '''
    def AddProtection(self):
        # UserID.
        UserID = self.userid

        # Protect player, and change the model color.
        myPlayer = getPlayer(UserID)
        myPlayer.setColor(255, 0, 0, 180)
        if str(dm_spawn_protection_type) == 'health':
            es.setplayerprop(UserID, 'CBasePlayer.m_iHealth', dm_spawn_protection_health)
        else:
            es.setplayerprop(UserID, 'CCSPlayer.baseclass.m_lifeState', 0)

        # Player is protected, just to know.
        self.protected = 1

        # Delay the spawn protection remover.
        delay.delayed(dm_spawn_protection_time, 'removeprotection%i' % UserID, self.RemProtection)

    '''
    | Removes spawn protection from a player.
    '''
    def RemProtection(self):
        # UserID.
        UserID = self.userid

        # UnProtect player, and change the model color.
        myPlayer = getPlayer(UserID)
        myPlayer.setColor(255, 255, 255, 255)
        if str(dm_spawn_protection_type) == 'health':
            es.setplayerprop(UserID, 'CBasePlayer.m_iHealth', 100) 
        else:
            es.setplayerprop(UserID, 'CCSPlayer.baseclass.m_lifeState', 512)

        # Player is not protected, just to know.
        self.protected = 0

    '''
    | Removes the players body and weapons when he get's killed.
    '''
    def PlayerDissolve(self):
        # UserID.
        UserID = self.userid

        # Dissolver mode.
        mode = random.randint(1,5) if int(dm_dissolver_type) == 6 else dm_dissolver_type

        # Remove this player weapons.
        for weapon in getWeaponList('#all'):
            # Remove all weapons of this type from the map
            for index in weapon.indexlist:
                # If the weapon has an owner, remove it
                if es.getindexprop(index,'CBaseEntity.m_hOwnerEntity') == -1:
                    spe.removeEntityByIndex(index)

        # Dissolve the body.
        if mode == 1:
            es.delayed(dm_dissolver_time, 'es_xfire %i cs_ragdoll kill' % UserID)
        else:       
            targetname = 'dissolve_%i' % UserID
            index = es.createentity('env_entity_dissolver', targetname)
            es.entitysetvalue(index, 'target', 'cs_ragdoll')
            es.entitysetvalue(index, 'magnitude', '1')
            es.entitysetvalue(index, 'dissolvetype', str(mode))
            es.spawnentity(index)
            es.delayed(dm_dissolver_time, 'es_xfire %i %s Dissolve'%(UserID, targetname))
''' ; end - PlayerManagement() ; '''

# Isto vai ser necessario para usar no addon, block-drop
def GetWeaponIndex(userid, weapon):
    returnvalue = -1
    for index in es.createentitylist(weapon):
        if (es.getindexprop(index, "CBaseCombatWeapon.m_hOwner") != es.getplayerhandle(userid)):
            continue
        returnvalue = index
    return returnvalue




delay = DelayManager()

'''
==============================================================================
   EVENTS
==============================================================================
'''

'''
| Event:  load()
|
|   ->  When the addon is loaded.
|
'''
def load():
    # Global variable.
    global Language

    # 
    es.doblock('corelib/noisy_on')

    # UserID.
    UserID = int(es.getuserid())

    # Load: Config File
    es.server.cmd('es_xmexec ../cfg/%s/config.cfg' % info['basename'])

    # Load: Language
    Language = langlib.Strings('%s/cfg/%s/language.ini' % (dmPath, info['basename']))

    # Load: Addons
    LoadAddons()

    # Register the chat filter.
    es.addons.registerSayFilter(ChatFilter)

    # End of the round.
    es.flags('remove', 'cheat', 'endround')

    # Set freezetime and roundtime to avoid gameplay interuptions and set friendly fire off.
    _cvars()

    # Reset all players settings
    for userid in es.getUseridList():
        Players[userid] = {}

    # Addon loaded, send message.
    msg(Language('load'))
''' ; end ; '''

'''
| Event:  unload()
|
|   ->  When the addon is unloaded.
|
'''
def unload():
    # Unregister the chat filter
    es.addons.unregisterSayFilter(ChatFilter)

    #
    es.doblock('corelib/noisy_off')

    # Unload all addons.
    for addon in LoadedAddons:
        es.server.queuecmd('es_unload %s/addons/%s' % (info['basename'], addon))

    # Delete Players dict
    del Players

    # Cancel the weapon remover loop.
    delay.cancelAll()

    # Addon unloaded, send message.
    msg(Language('unload'))
''' ; end ; '''

'''
| Event: es_map_start()
|
|   ->  When a map is started.
|
'''
def es_map_start(ev):
    # Global variable.
    global CurMap

    # Cancel all delayed actions.
    delay.cancelAll()

    # Save the map name.
    CurMap = ev['mapname']    

    # Load: Map Config
    LoadMap()

    # Start the idle weapon removal loop
    WeaponRemoverLoop()

    # Create Popups.
    PopupsCreate()

    # Set freezetime and roundtime to avoid gameplay interuptions and set friendly fire off.
    _cvars()

    # 
    for userid in es.getUseridList():
        del Players[userid]
        Players[userid] = PlayerManagement(userid)
''' ; end - es_map_start() ; '''

'''
| Event:  round_start()
|
|   ->  When a round starts.
|
'''
def round_start(ev):
    # Global variables.
    global AllowRespawn

    # Enable respawn.
    AllowRespawn = True

    # UserID.
    UserID = int(es.getuserid())

    # Remove all idle weapons that exists on the map.
    WeaponRemoverLoop()

    # Disable End of the round?
    if int(dm_allow_roundend) == 0:
        roundtime = es.ServerVar('mp_roundtime') * 60 + es.ServerVar('mp_freezetime')
        delay.delayed(roundtime, 'round_end', es.server.queuecmd, ('endround', ))

    # Disable buy zones?
    if int(dm_allow_buyzones) == 0:
        es.server.queuecmd('es_xfire %i func_buyzone Disable' % UserID)
    else:
        es.server.queuecmd('es_xfire %i func_buyzone Enable' % UserID)

    # Disable Map Objectives?
    if int(dm_allow_map_objectives) == 0:  
        es.server.queuecmd('es_xfire %i func_hostage_rescue kill' % UserID)
        es.server.queuecmd('es_xfire %i hostage_entity kill' % UserID)
        es.server.queuecmd('es_xfire %i func_bomb_target kill'%  UserID)
''' ; end - round_start() ; '''

'''
| Event: round_end()
|
|   ->  When a round ends.
|
'''
def round_end(ev):
    # Disable respawn.
    global AllowRespawn
    AllowRespawn = False

    # Cancel all delayed actions.
    delay.cancelAll()
''' ; end - round_end() ; '''

'''
| Event:  player_activate()
|
|   ->  When a player joins the server.
|
'''
def player_activate(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Add player to the dict.
    if UserID not in Players:
        Players[UserID] = PlayerManagement(UserID)
    else:
        Players[UserID].Reset()
''' ; end - player_activate() ; '''

'''
| Event:  player_disconnect()
|
|   ->  When a player leaves the server.
|
'''
def player_disconnect(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Delete the player from the dict.
    if UserID in Players:
        # Delete the player.
        del Players[UserID]

        # Cancel this player delays.
        delay.cancel('removeprotection%i' % UserID)
        delay.cancel('respawn%i' % UserID)
''' ; end - player_disconnect() ; '''

'''
| Event:  player_spawn()
|
|   ->  When a player spawns.
|
'''
def player_spawn(ev):
    # Variables.
    UserID = int(ev['userid'])
    Team   = int(ev['es_userteam'])

    # If the player is on T or CT.
    if Team >= 2 and UserID in Players:
        # Player team.
        Players[UserID].team = Team

        # Spawn the player.
        Players[UserID].Spawn()
''' ; end - player_spawns() ; '''

'''
| Event: player_team()
|
|   ->  When a player chooses or swap's team.
|
'''
def player_team(ev):
    # Variable.
    UserID = int(ev['userid'])
    Team   = int(ev['team'])

    # If the team is T or CT.
    if Team >= 2 and UserID in Players:
        # Spawn the player.
        delay.delayed(dm_spawn_deadtime, 'respawn%i' % UserID, Respawn, (UserID, ))
''' ; end - player_team() ; '''

'''
| Event:  player_death()
|
|   ->  When a player dies.
|
'''
def player_death(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Allow the use of guns menu.
    if UserID in Players:
        Players[UserID].used = 0

    # Respawn the player if the round hasn't ended
    if AllowRespawn:
        # Respawn the player.
        delay.delayed(dm_spawn_deadtime, 'respawn%i' % UserID, Respawn, (UserID, ))
        
    # Remove the dead body.
    Players[UserID].PlayerDissolve()
''' ; end - player_death() ; '''

'''
| Event:  player_hurt()
|
|   ->  When a player get's hurt.
|
'''
def player_hurt(ev):
    # UserID.
    UserID = int(ev['userid'])

    # If the player is still protected by spawn protection.
    if Players[UserID].protected == 1:
        es.setplayerprop(UserID, 'CBasePlayer.m_iHealth', dm_spawn_protection_health)
''' ; end - player_hurt() ; '''

'''
| Event:  flashbang_detonate()
|
|   ->  When a FlashBang Grenade detonates.
'''
def flashbang_detonate(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Unlimited nades?
    if int(dm_give_unlimitednade) == 1:
        # Give more Flash's
        if not es.getplayerprop(UserID, 'CBasePlayer.pl.deadflag'):
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_flashbang' % UserID)
''' ; end - flashbang_detonate() ; '''

'''
| Event:  smokegrenade_detonate()
|
|   ->  When a Smoke Grenade detonates.
|
'''
def smokegrenade_detonate(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Unlimited nades?
    if int(dm_give_unlimitednade) == 1:
        # Give more Smokes's
        if not es.getplayerprop(UserID, 'CBasePlayer.pl.deadflag'):
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_smokegrenade' % UserID)
''' ; end - smokegrenade_detonate() ; '''

'''
| Event:  hegrenade_detonate()
|
|   ->  When a HE Grenade detonates.
|
'''
def hegrenade_detonate(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Unlimited nades?
    if int(dm_give_unlimitednade) == 1:
        # Give more HE's
        if not es.getplayerprop(UserID, 'CBasePlayer.pl.deadflag'):
            gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_hegrenade' % UserID)
''' ; end - hegrenade_detonate() ; '''

'''
| Event:  weapon_reload()
|
|   ->  When a player reloads a weapon.
|
'''
def weapon_reload(ev):
    # UserID.
    UserID = int(ev['userid'])

    # Unlimited ammo?
    if int(dm_give_unlimitedammo) == 1:
        # Give the player more ammo then.
        myPlayer = getPlayer(UserID)
        weapon = myPlayer.attributes['weapon']
        if weapon in Weapons:
            myPlayer.set('ammo',(weapon, Weapons[weapon]))
''' ; end - weapon_reload() ; '''


'''
==============================================================================
   FUNCTIONS
==============================================================================
'''

'''
| Function
'''
def changeTeam(userid, team, kill=0):
    if kill:
        es.server.queuecmd('es_xchangeteam %s %s'%(userid, team))
    else:
        cstrike.switchTeam(int(userid), team)
''' ; end - changeTeam() ; '''

'''
| Func: PlayerDead()
|
|   ->  This function checks if a player is alive.
|
'''
def PlayerDead(userid):
    return es.getplayerprop(userid, 'CBasePlayer.pl.deadflag')
''' ; end - PlayerDead() ; '''

'''
| Func: msg()
|
|   ->  Send messages to everyone on the server.
|
'''
def msg(text):
    es.msg('#multi', '#green[\x05DM#green] #lightgreen%s' % text)
''' ; end - msg() ; '''

'''
| Func: tell()
|
|   ->  Send a message to a specific player on the server.
|
'''
def tell(userid, text):
    es.tell(userid, '#multi', '#green[\x05DM#green] #lightgreen%s' % text)
''' ; end - tell() ; '''

'''
| Func: HudHint()
|
|   ->  Send a message in the middle of the screen.
|
'''
def centermsg(text, userid = 0):
    if int(userid) == 0:
        es.centermsg(text)
    else:
        es.centertell(userid, text)
''' ; end - HudHint() ; '''

'''
| Func: KeyHintText()
|
|   ->  Send a message in to the right of the screen.
|
'''
def KeyHintText(userid, text):
    es.usermsg('create', 'mymsg', 'KeyHintText')
    es.usermsg('write', 'byte', 'mymsg', 1)
    es.usermsg('write', 'string', 'mymsg', text)
    es.usermsg('send', 'mymsg', userid)
    es.usermsg('delete', 'mymsg')
''' ; end - KeyHintText() ; '''

'''
| Func: _cvars()
|
|   ->  This function set's some cvars to have a better gameplay.
|
'''
def _cvars():
    # Set freezetime and roundtime to avoid gameplay interuptions and set friendly fire off.
    gamethread.delayed(3, es.server.queuecmd, 'mp_freezetime 0')
    gamethread.delayed(3, es.server.queuecmd, 'mp_roundtime 9')
    gamethread.delayed(3, es.server.queuecmd, 'mp_friendlyfire 0')
    gamethread.delayed(3, es.server.queuecmd, 'mp_restartgame 1')
''' ; end - _cvars() ; '''


def LoadMap():
    # Global variables.
    global MapsCfg

    # General Settings
    global dm_enable, dm_guns_menu, dm_weapon_remover

    # Map settings
    global dm_allow_roundend, dm_allow_buyzones, dm_allow_map_objectives

    # Weapon settings
    global dm_p_weapons, dm_s_weapons, dm_restrict

    # Global Equipment variables.
    global dm_amount_health, dm_amount_armor, dm_give_hegrenade, dm_give_flashbang, dm_give_smokegrenade, dm_give_unlimitednade, dm_give_unlimitedammo, dm_give_ngvision

    # File with map config.
    file = '%s/cfg/%s/maps/%s.ini' % (dmPath, info['basename'], CurMap)

    # Execute original config.
    es.server.cmd('es_xmexec ../cfg/%s/config.cfg' % info['basename'])

    # Update cvars.
    dm_enable                   = es.ServerVar('dm_enable')
    dm_guns_menu                = es.ServerVar('dm_guns_menu')
    dm_weapon_remover           = es.ServerVar('dm_weapon_remover')
    dm_allow_roundend           = es.ServerVar('dm_allow_roundend')
    dm_allow_buyzones           = es.ServerVar('dm_allow_buyzones')
    dm_allow_map_objectives     = es.ServerVar('dm_allow_map_objectives')
    dm_p_weapons                = es.ServerVar('dm_p_weapons')
    dm_s_weapons                = es.ServerVar('dm_s_weapons')
    dm_restrict                 = es.ServerVar('dm_restrict')
    dm_amount_health            = es.ServerVar('dm_amount_health')
    dm_amount_armor             = es.ServerVar('dm_amount_armor')
    dm_give_hegrenade           = es.ServerVar('dm_give_hegrenade')
    dm_give_flashbang           = es.ServerVar('dm_give_flashbang')
    dm_give_smokegrenade        = es.ServerVar('dm_give_smokegrenade')
    dm_give_unlimitednade       = es.ServerVar('dm_give_unlimitednade')
    dm_give_unlimitedammo       = es.ServerVar('dm_give_unlimitedammo')
    dm_give_ngvision            = es.ServerVar('dm_give_ngvision')
    dm_spawn_method             = es.ServerVar('dm_spawn_method')
    dm_spawn_deadtime           = es.ServerVar('dm_spawn_deadtime')
    dm_spawn_protection         = es.ServerVar('dm_spawn_protection')
    dm_spawn_protection_time    = es.ServerVar('dm_spawn_protection_time')
    dm_spawn_protection_type    = es.ServerVar('dm_spawn_protection_type')
    dm_spawn_protection_alpha   = es.ServerVar('dm_spawn_protection_alpha')
    dm_spawn_protection_health  = es.ServerVar('dm_spawn_protection_health')
    dm_dissolver_time           = es.ServerVar('dm_dissolver_time')
    dm_dissolver_type           = es.ServerVar('dm_dissolver_type')

    # Check if map config exists.
    if not os.path.isfile(file):
        MapsCfg = None
        return

    # Load map config.
    MapsCfg = ConfigObj(cfglib.AddonINI(file))

    # Update Cvars AGAIN!!
    # GENERAL SETTINGS
    dm_enable               = 1 if MapsCfg['settings']['enable_deathmatch'] == 'yes' else 0
    dm_guns_menu            = 1 if MapsCfg['settings']['guns_menu'] == 'yes' else 0
    dm_weapon_remover       = 1 if MapsCfg['settings']['weapon_remover'] == 'yes' else 0

    #
    dm_allow_roundend       = 1 if MapsCfg['map']['enable_round_end'] == 'yes' else 0
    dm_allow_buyzones       = 1 if MapsCfg['map']['enable_buy_zones'] == 'yes' else 0
    dm_allow_map_objectives = 1 if MapsCfg['map']['enable_objectives'] == 'yes' else 0

    # WEAPONS SETTINGS
    dm_p_weapons            = MapsCfg['weapons']['p_weapons'] if MapsCfg['weapons']['p_weapons'] != '' else dm_p_weapons
    dm_s_weapons            = MapsCfg['weapons']['s_weapons'] if MapsCfg['weapons']['s_weapons'] !='' else dm_s_weapons

    # EQUIPMENT SETTINGS...
    dm_amount_health        = MapsCfg['equipment']['health']
    dm_amount_armor         = MapsCfg['equipment']['armor']
    dm_give_flashbang       = MapsCfg['equipment']['flashbangs'] if MapsCfg['equipment']['flashbangs'] >= 1 else 0
    dm_give_smokegrenade    = 1 if MapsCfg['equipment']['smokegrenade'] == 'yes' else 0
    dm_give_hegrenade       = 1 if MapsCfg['equipment']['hegrenade'] == 'yes' else 0
    dm_give_ngvision        = 1 if MapsCfg['equipment']['ngvision'] == 'yes' else 0
    dm_give_unlimitedammo   = 1 if MapsCfg['equipment']['unlimitedammo'] == 'yes' else 0
    dm_give_unlimitednade   = 1 if MapsCfg['equipment']['unlimitednade'] == 'yes' else 0


    # ADDON SETTINGS
    # Load Addons:
    if MapsCfg['addons']['load'] != '':
        for addon in MapsCfg['addons']['load'].split():
            if addon not in LoadedAddons:
                es.server.queuecmd('es_xload %s/addons/%s' % (info['basename'], addon))

    # UnLoad Addons:
    if MapsCfg['addons']['unload'] != '':
        for addon in MapsCfg['addons']['unload'].split():
            if addon not in LoadedAddons:
                es.server.queuecmd('es_xunload %s/addons/%s' % (info['basename'], addon))
''' ; end - LoadMap() ; '''

'''
| Func: WeaponRemoverLoop()
|
|   ->  Creates a loop to keep removing dropped weapons from the map.
|
'''
def WeaponRemoverLoop():
    #
    if MapsCfg != None and MapsCfg['settings']['weapon_remover'] == 'yes':
        for weapon in getWeaponList('#all'):
            # Remove all weapons of this type from the map
            for index in weapon.indexlist:
                # If the weapon has an owner, stop here
                if es.getindexprop(index,'CBaseEntity.m_hOwnerEntity') != -1:
                    continue

                spe.removeEntityByIndex(index)

    #
    elif MapsCfg != None and MapsCfg['settings']['weapon_remover'] == 'no':
        return

    #
    else:
        for weapon in getWeaponList('#all'):
            # Remove all weapons of this type from the map
            for index in weapon.indexlist:
                # If the weapon has an owner, stop here
                if es.getindexprop(index,'CBaseEntity.m_hOwnerEntity') != -1:
                    continue

                spe.removeEntityByIndex(index)

    # Create the delay.
    delay.delayed(5, "WeaponRemoverLoop", WeaponRemoverLoop)
''' ; end - WeaponRemoverLoop() ; '''

'''
| Func:  Respawn()
|
|   ->  Respawns a player.
|
'''
def Respawn(userid):
    UserID = int(userid)

    if es.getplayerprop(UserID, 'CBasePlayer.pl.deadflag'):
        es.setplayerprop(UserID, 'CCSPlayer.m_iPlayerState', 0)
        es.setplayerprop(UserID, 'CCSPlayer.baseclass.m_lifeState', 512)
        es.spawnplayer(UserID)
''' ; end - Respawn() ; '''

'''
| Func:  ChatFilter()
|
|   ->  Chat filter, the name says all.
|
'''
def ChatFilter(uid, text, teamonly):
    # Variables
    text     = text.strip('"')
    UserID   = int(uid)
    SteamID  = es.getplayersteamid(UserID)
    Username = es.getplayername(UserID)

    # To open the guns menu.
    if text.lower() in [ 'guns', '!guns' ]:
        if int(dm_guns_menu) == 1:
            if Players[UserID].used == 0:
                myPopups['dm_gunsmenu'].unsend(UserID)
                gamethread.delayed(0.1, myPopups['dm_gunsmenu'].send, UserID)
            else:
                tell(UserID, Language('gunsmenu used'))
        else:
            tell(UserID, Language('gunsmenu off'))
        return(0, None, None)

    # Other text... return it :)
    return(UserID, text, teamonly)
''' ; end - ChatFilter() '''

'''
| Func: LoadAddons()
|
|   ->  This function is used to load addons.
|
'''
def LoadAddons():
    # File with addons.
    addonsList = 'cfg/%s/addons.txt' % info['basename']

    # Handle file. #
    file = open('%s/%s' % (dmPath, addonsList), 'rb')
    addons = file.readlines()
    file.close()

    # Load the addons
    for addon in addons:
        addon = addon.strip()
        comment = addon.find('//')
        if comment > -1: addon = addon[:comment]
        comment = addon.find('#')
        if comment > -1: addon = addon[:comment]
        if addon:
            es.server.queuecmd('es_xload %s/addons/%s' % (info['basename'], addon))
            LoadedAddons.append(addon)
''' ; end - LoadAddons() ; '''



'''
==============================================================================
   POPUPS
==============================================================================
'''
def PopupsCreate():
    # UserID
    #UserID = int(es.getuserid())

    # Guns menu
    myPopups['dm_gunsmenu'] = popuplib.create('dm_gunsmenu')
    myPopups['dm_gunsmenu'].addline('DeathMatch')
    myPopups['dm_gunsmenu'].addline(' - Choose an option:')
    myPopups['dm_gunsmenu'].addline('-------------------------')
    myPopups['dm_gunsmenu'].addline('->1. New weapons')
    myPopups['dm_gunsmenu'].addline('->2. Same weapons')
    myPopups['dm_gunsmenu'].addline('->3. Same weapons every time')
    myPopups['dm_gunsmenu'].addline('->4. Random weapons')
    myPopups['dm_gunsmenu'].addline('->5. Random weapons every time')
    myPopups['dm_gunsmenu'].addline('-------------------------')
    myPopups['dm_gunsmenu'].addline('0. Exit')
    myPopups['dm_gunsmenu'].menuselect = PopupGest

    # Primary weapons.
    myPopups['dm_gunsmenupwep'] = popuplib.easymenu('dm_gunsmenupwep', None, PopupGest)
    myPopups['dm_gunsmenupwep'].settitle('DeathMatch')
    myPopups['dm_gunsmenupwep'].setdescription(' - Select your Primary Weapon')
    for c in str(dm_p_weapons).split():
        # Restrict weapons from the: Map config.
        if MapsCfg != None and MapsCfg['weapons']['restrict'] != '':
            if c.lower() not in MapsCfg['weapons']['restrict'].split():
                myPopups['dm_gunsmenupwep'].addoption(c, c)

        # Restrict weapons from the: Config file.
        elif c.lower() not in str(dm_restrict).split():
            myPopups['dm_gunsmenupwep'].addoption(c, c)

    # Secondary weapons.
    myPopups['dm_gunsmenuswep'] = popuplib.easymenu('dm_gunsmenuswep', None, PopupGest)
    myPopups['dm_gunsmenuswep'].settitle('DeathMatch')
    myPopups['dm_gunsmenuswep'].setdescription(' - Select your Secondary Weapon')
    for c in str(dm_s_weapons).split():
        # Restrict weapons from the: Map config.
        if MapsCfg != None and MapsCfg['weapons']['restrict'] != '':
            if c.lower() not in MapsCfg['weapons']['restrict'].split():
                myPopups['dm_gunsmenuswep'].addoption(c, c)

        # Restrict weapons from the: Config file.
        elif c.lower() not in str(dm_restrict).split():
            myPopups['dm_gunsmenuswep'].addoption(c, c)
''' ; end - PopupsCreate() ; '''



def PopupGest(userid, choice, popup):
    # Parameters.
    UserID = int(userid)

    # ~~~~~~~~~~~~~~~~
    # Guns Menu.
    if popup == 'dm_gunsmenu':
        if choice == 10:
            myPopups['dm_gunsmenu'].send(UserID)

        # 1: New weapons
        elif choice == 1:
            # Send secondary weapons popups.
            myPopups['dm_gunsmenuswep'].unsend(UserID)
            myPopups['dm_gunsmenuswep'].send(UserID)

            # Save menu selection.
            Players[UserID].gunmenu = 1

        # 2: Same weapons
        elif choice == 2:
            if Players[UserID].pweapon and Players[UserID].sweapon:
                # Give weapons.
                Players[UserID].GiveWeapons()

                # Player used guns menu, needs to wait for next respawn to use it again.
                Players[UserID].used = 1
            else:
                # Send guns menu.
                myPopups['dm_gunsmenu'].unsend(UserID)
                gamethread.delayed(0.1, myPopups['dm_gunsmenu'].send, UserID)

        # 3: Same weapons every time
        elif choice == 3:
            if Players[UserID].pweapon and Players[UserID].sweapon:
                # Give weapons.
                Players[UserID].GiveWeapons()

                # Save menu selection.
                Players[UserID].gunmenu = 2
            else:
                # Send guns menu.
                myPopups['dm_gunsmenu'].unsend(UserID)
                gamethread.delayed(0.1, myPopups['dm_gunsmenu'].send, UserID)

        # 4: Random weapons
        elif choice == 4:
            # Give random weapons.
            Players[UserID].GiveWeaponsRandom()

            # Player used guns menu, needs to wait for next respawn.
            Players[UserID].used = 1

        # 5: Random weapons every time
        elif choice == 5:
            # Give random weapons.
            Players[UserID].GiveWeaponsRandom()

            # Save menu selection.
            Players[UserID].gunmenu = 3

    # ~~~~~~~~~~~~~~~~
    # Primary weapons.
    elif popup == 'dm_gunsmenupwep':
        # Save the choice.
        Players[UserID].pweapon = choice

        # Remove primary weapons, if needed.
        Players[UserID].RemoveWeapons('primary')

        # Give the weapon.
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, choice))

        # Player used guns menu, needs to wait for next respawn.
        Players[UserID].used = 1

    # ~~~~~~~~~~~~~~~~
    # Secondary weapons.
    elif popup == 'dm_gunsmenuswep':
        # Save the choice.
        Players[UserID].sweapon = choice

        # Remove secondary weapons, if needed.
        Players[UserID].RemoveWeapons('secondary')

        # Give the weapon.
        gamethread.delayed(0.1, es.server.queuecmd, 'es_xgive %i weapon_%s' % (UserID, choice))

        # Player used guns menu, needs to wait for next respawn.
        Players[UserID].used = 1

        # Show secondary weapons menu.
        myPopups['dm_gunsmenupwep'].send(UserID)
''' ; end - PopupGest() ; '''